function KanbanController($scope, $rootScope, $timeout, $filter, ScreenService) {
	var self = this;
	var container = $('.zh-kanban-container');
	self.i18n = ScreenService.i18n;

	var widgetAccessorDescriptor = {
		get: function () {
			return $scope.widget;
		}
	};

	Object.defineProperty(self, "widget", widgetAccessorDescriptor);

	// Floating Control defaults
	var floatingControlDefaults = {
		pdfAction: false,
		xlsAction: false,
		searchAction: true,
		csvAction: false,
		customizationAction: {
			filter: false,
			layout: false,
			view: false
		}
	};
	self.widget.floatingControl = self.widget.floatingControl !== false && Util.extend(floatingControlDefaults, self.widget.floatingControl || {});

	function extendBehavior() {
		Util.decorateFromConstructor(self, OrderByable);
		Util.decorateFromConstructor(self, Searchable);
	}
	extendBehavior();

	$scope.originColumn = null;
	$scope.destinationColumn = null;
	$scope.sortableOptions = {
		placeholder: "kanban-placeholder",
		connectWith: ".kanban-item-container",
		handle: '.kanban-handle',
		beforeStop: function(e, ui) {
			$scope.destinationColumn = $(ui.item).parents('.kanban-column').attr("column-id");
			if ($scope.widget.beforeMoveItem) {
				var event = $scope.widget.beforeMoveItem({
					"item": ui.item.sortable.model,
					"columns": {
						"from": $scope.originColumn,
						"to": $scope.destinationColumn
					},
					"lines": {
						"from": ui.item.sortable.index,
						"to": ui.item.index()
					}
				});
				if (event.data === false) {
					ui.item.sortable.cancel();
				}
			}
		},
		start: function(e, ui) {
			$scope.originColumn = $(e.target).parents('.kanban-column').attr("column-id");
			container.removeClass('nonbounce'); // prevent scroll during dragging
		},
		sort: function(e, ui) {
			if (ui.helper.offset().left < 0) {
				ui.helper.css('left', ui.helper.position().left - container.scrollLeft());
				container.scrollLeft(0);
			}
			if (ui.helper.offset().left + ui.helper[0].clientWidth > container.width()) {
				ui.helper.css('left', ui.helper.position().left + (container.find('.zh-kanban-column-container').width() - container.width() - container.scrollLeft()));
				container.scrollLeft(container.find('.zh-kanban-column-container').width() - container.width());
			}
		},
		stop: function(e, ui) {
			var itemModel = ui.item.sortable.model;
			container.addClass('nonbounce'); // prevent scroll during dragging
			if (!ui.item.sortable.isCanceled()) {
				var destinationColumn = $scope.destinationColumn;
				var fromColumn = ui.item.sortable.model.column;
				ui.item.sortable.model.column = $scope.destinationColumn;
				destinationColumn = getColumnIndexInArray(Number.parseInt(destinationColumn));
				fromColumn = getColumnIndexInArray(Number.parseInt(fromColumn));
				var fromItems = self.widget.columns[fromColumn].items;
				var destinationItems = self.widget.columns[destinationColumn].items;
				self.widget.columns[destinationColumn].items = self.orderByFilter(destinationItems);
				self.widget.columns[fromColumn].items = self.orderByFilter(fromItems);
				if ($scope.widget.afterMoveItem) {
					$scope.widget.afterMoveItem({
						"item": ui.item.sortable.model,
						"columns": {
							"from": $scope.originColumn,
							"to": $scope.destinationColumn
						},
						"lines": {
							"from": ui.item.sortable.index,
							"to": ui.item.index(),
						}
					});
				}
			}
		}
	};

	function getColumnIndexInArray(id) {
		var index;
		self.widget.columns.every(function(col, idx) {
			if (col.id !== id) {
				return true;
			} else {
				index = idx;
			}
		});
		return index;
	}
	// Toggle functions
	function deselectAll(widget, column) {
		widget = widget || self.widget;
		if (column) {
			column.items.forEach(function(row) {
				row.__isSelected = false;
				widget.dataSource.removeCheckedRows(row);
			});
		} else {
			var event = widget.unSelectAll();
			if (!event || !event.isDefaultPrevented()) {
				widget.dataSource.clearCheckedRows();
			}
		}
		if (widget.dataSource.checkedRows.length === 0) {
			if (!$scope._currentPopup) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}
		}
	}

	function selectAll(widget, column) {
		widget = widget || self.widget;
		column.items.forEach(function(row) {
			if (!row.__notSelectable) {
				row.__isSelected = true;
				widget.dataSource.addCheckedRows(row);
			}
		});
		if (!$scope._currentPopup) {
			$rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
		}
	}

	var uiDeselectAllDeregister = $rootScope.$on("ui.deselectAll", function(event, widget) {
		deselectAll(widget);
	});

	var _widget = self.widget;
	$scope.$on('$destroy', function() {
		uiDeselectAllDeregister();

		delete _widget.toggle;
		delete _widget.setSearchCriteria;
	});

	function getIndex(row, scope) {
		return self.widget.dataSource.checkedRows.indexOf(row);
	}

	this.toggle = function(row) {
		var widget = self.widget;
		widget.selectedRow = row;
		var event;
		if (widget.beforeSelectRow) {
			event = widget.beforeSelectRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			var index = getIndex(row, $scope);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}

			if (!$scope._currentPopup) {
				if (!widget.dataSource.checkedRows.length) {
					$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
				}
			}
			widget.updateCheckedRows();

			if (widget.afterSelectRow) {
				widget.afterSelectRow();
			}
		}
	};

	this.widget.toggle = this.toggle;

	this.hasCheckedRows = function(column) {
		return this.getCheckedRows(column).length > 0;
	};

	this.getCheckedRows = function(column) {
		return this.widget.dataSource.checkedRows.filter(function(item) {
			return item.column == column.id;
		});
	};

	this.toggleAll = function(column) {
		if (this.hasCheckedRows(column)) {
			deselectAll(this.widget, column);
		} else {
			selectAll(this.widget, column);
		}
		this.widget.updateCheckedRows();
	};

	function clearSearchCriteria() {
		self.widget.searchCriteria = {};
		self.widget.searchList = "";
	}

	var searchFromSearchCriteria = Util.buildDebounceMethod(function() {
		if (!self.widget._noReloadFromSearchCriteria) {
			self.widget.searchList = self.widget.searchCriteria && self.widget.searchCriteria._ALL || undefined;
		}
		self.widget._noReloadFromSearchCriteria = false;
	}, 200);

	function setSearchCriteria(searchCriteria) {
		self.widget._noReloadFromSearchCriteria = true;

		clearSearchCriteria();
		if (searchCriteria) {
			searchCriteria.forEach(function(field) {
				var fieldValue = field.value;
				if (fieldValue[fieldValue.length - 1] == '%') {
					fieldValue = fieldValue.substring(0, (fieldValue.length - 1));
				}
				if (fieldValue[0] == "%") {
					fieldValue = fieldValue.substring(1, fieldValue.length);
				}
				self.widget.searchCriteria[field.name] = fieldValue;
			});
		}

		searchFromSearchCriteria(self.widget.searchCriteria);
	}
	this.widget.setSearchCriteria = setSearchCriteria;

	// Search functions
	this.search = function(gridRow) {
		self.widget.searchList = $scope.widget.searchCriteria ? $scope.widget.searchCriteria._ALL : "";
		var rowVisibility = true;
		if (Util.objectHasAnyValue($scope.widget.searchCriteria)) {
			var filterConfig = {
				ignoreUnexistentFields: true
			};
			rowVisibility = self.checkRowVisibilityFromCriteria(gridRow, $scope.widget.searchCriteria, filterConfig);
		}
		gridRow.__notSelectable = !rowVisibility;
		return rowVisibility;
	};

	this.getColumnItems = function(column) {
		return (column.items || []).filter(this.search);
	};

	function updateColumns() {
		if (self.widget.dataSource.data) {
			self.widget.columns.forEach(function(column) {
				column.items = self.widget.dataSource.data.filter(function setColumnByFilterAndId(row) {
					if(self.widget.searchCriteria){
						return row.column == column.id && self.search(row);
					} else {
						return row.column == column.id;
					}
				});

				column.items = self.orderByFilter(column.items);
			});
		}
	}

	this.orderByFilter = function(items) {
		self.updateOrderByColumns($scope.widget);
		var oderByColumns = self.getOrderByColumns();
		var orderedItems;
		//Prevent angular's filter from rearranging items even when there is no sort key defined
		if (oderByColumns && oderByColumns.length) {
			orderedItems = $filter('orderBy')(items, self.getOrderByColumns());
		} else {
			orderedItems = items;
		}

		return orderedItems;
	};

	$scope.$watchCollection('widget.dataSource.data', function() {
		$timeout(function() {
			updateColumns();
			$scope.resizeItems();
		}, 250);
	});

	$scope.$watchCollection(function() { return $scope.widget.searchCriteria; }, function(current, old) {
		if (!angular.equals(current, old)) {
			updateColumns();
		}
	});

}